@@ -17,15 +17,14 @@
 package kafka.coordinator.group
 
 import java.util.UUID
-import java.util.concurrent.locks.ReentrantLock
 
 import kafka.common.OffsetAndMetadata
-import kafka.utils.{CoreUtils, Logging, nonthreadsafe}
+import kafka.utils.{Logging, nonthreadsafe}
 import org.apache.kafka.common.TopicPartition
 
 import scala.collection.{Seq, immutable, mutable}
 
-private[group] sealed trait GroupState
+private[group] sealed trait GroupState { def state: Byte }
 
 /**
  * Group is preparing to rebalance
@@ -36,11 +35,11 @@ private[group] sealed trait GroupState
  *         park join group requests from new or existing members until all expected members have joined
  *         allow offset commits from previous generation
  *         allow offset fetch requests
- * transition: some members have joined by the timeout => CompletingRebalance
+ * transition: some members have joined by the timeout => AwaitingSync
  *             all members have left the group => Empty
  *             group is removed by partition emigration => Dead
  */
-private[group] case object PreparingRebalance extends GroupState
+private[group] case object PreparingRebalance extends GroupState { val state: Byte = 1 }
 
 /**
  * Group is awaiting state assignment from the leader
@@ -55,7 +54,7 @@ private[group] case object PreparingRebalance extends GroupState
  *             member failure detected => PreparingRebalance
  *             group is removed by partition emigration => Dead
  */
-private[group] case object CompletingRebalance extends GroupState
+private[group] case object AwaitingSync extends GroupState { val state: Byte = 5}
 
 /**
  * Group is stable
@@ -71,7 +70,7 @@ private[group] case object CompletingRebalance extends GroupState
  *             follower join-group with new metadata => PreparingRebalance
  *             group is removed by partition emigration => Dead
  */
-private[group] case object Stable extends GroupState
+private[group] case object Stable extends GroupState { val state: Byte = 3 }
 
 /**
  * Group has no more members and its metadata is being removed
@@ -84,7 +83,7 @@ private[group] case object Stable extends GroupState
  *         allow offset fetch requests
  * transition: Dead is a final state before group metadata is cleaned up, so there are no transitions
  */
-private[group] case object Dead extends GroupState
+private[group] case object Dead extends GroupState { val state: Byte = 4 }
 
 /**
   * Group has no more members, but lingers until all offsets have expired. This state
@@ -101,32 +100,16 @@ private[group] case object Dead extends GroupState
   *             group is removed by partition emigration => Dead
   *             group is removed by expiration => Dead
   */
-private[group] case object Empty extends GroupState
+private[group] case object Empty extends GroupState { val state: Byte = 5 }
 
 
 private object GroupMetadata {
   private val validPreviousStates: Map[GroupState, Set[GroupState]] =
-    Map(Dead -> Set(Stable, PreparingRebalance, CompletingRebalance, Empty, Dead),
-      CompletingRebalance -> Set(PreparingRebalance),
-      Stable -> Set(CompletingRebalance),
-      PreparingRebalance -> Set(Stable, CompletingRebalance, Empty),
+    Map(Dead -> Set(Stable, PreparingRebalance, AwaitingSync, Empty, Dead),
+      AwaitingSync -> Set(PreparingRebalance),
+      Stable -> Set(AwaitingSync),
+      PreparingRebalance -> Set(Stable, AwaitingSync, Empty),
       Empty -> Set(PreparingRebalance))
-
-  def loadGroup(groupId: String,
-                initialState: GroupState,
-                generationId: Int,
-                protocolType: String,
-                protocol: String,
-                leaderId: String,
-                members: Iterable[MemberMetadata]): GroupMetadata = {
-    val group = new GroupMetadata(groupId, initialState)
-    group.generationId = generationId
-    group.protocolType = if (protocolType == null || protocolType.isEmpty) None else Some(protocolType)
-    group.protocol = Option(protocol)
-    group.leaderId = Option(leaderId)
-    members.foreach(group.add)
-    group
-  }
 }
 
 /**
@@ -167,35 +150,33 @@ case class CommitRecordMetadataAndOffset(appendedBatchOffset: Option[Long], offs
  *  3. leader id
  */
 @nonthreadsafe
-private[group] class GroupMetadata(val groupId: String, initialState: GroupState) extends Logging {
-  private[group] val lock = new ReentrantLock
+private[group] class GroupMetadata(val groupId: String, initialState: GroupState = Empty) extends Logging {
 
   private var state: GroupState = initialState
-  var protocolType: Option[String] = None
-  var generationId = 0
-  private var leaderId: Option[String] = None
-  private var protocol: Option[String] = None
 
   private val members = new mutable.HashMap[String, MemberMetadata]
+
   private val offsets = new mutable.HashMap[TopicPartition, CommitRecordMetadataAndOffset]
+
   private val pendingOffsetCommits = new mutable.HashMap[TopicPartition, OffsetAndMetadata]
+
   private val pendingTransactionalOffsetCommits = new mutable.HashMap[Long, mutable.Map[TopicPartition, CommitRecordMetadataAndOffset]]()
+
   private var receivedTransactionalOffsetCommits = false
+
   private var receivedConsumerOffsetCommits = false
 
+  var protocolType: Option[String] = None
+  var generationId = 0
+  var leaderId: String = null
+  var protocol: String = null
   var newMemberAdded: Boolean = false
 
-  def inLock[T](fun: => T): T = CoreUtils.inLock(lock)(fun)
-
   def is(groupState: GroupState) = state == groupState
   def not(groupState: GroupState) = state != groupState
   def has(memberId: String) = members.contains(memberId)
   def get(memberId: String) = members(memberId)
 
-  def isLeader(memberId: String): Boolean = leaderId.contains(memberId)
-  def leaderOrNull: String = leaderId.orNull
-  def protocolOrNull: String = protocol.orNull
-
   def add(member: MemberMetadata) {
     if (members.isEmpty)
       this.protocolType = Some(member.protocolType)
@@ -204,18 +185,18 @@ private[group] class GroupMetadata(val groupId: String, initialState: GroupState
     assert(this.protocolType.orNull == member.protocolType)
     assert(supportsProtocols(member.protocols))
 
-    if (leaderId.isEmpty)
-      leaderId = Some(member.memberId)
+    if (leaderId == null)
+      leaderId = member.memberId
     members.put(member.memberId, member)
   }
 
   def remove(memberId: String) {
     members.remove(memberId)
-    if (isLeader(memberId)) {
+    if (memberId == leaderId) {
       leaderId = if (members.isEmpty) {
-        None
+        null
       } else {
-        Some(members.keys.head)
+        members.keys.head
       }
     }
   }
@@ -274,11 +255,11 @@ private[group] class GroupMetadata(val groupId: String, initialState: GroupState
     assert(notYetRejoinedMembers == List.empty[MemberMetadata])
     if (members.nonEmpty) {
       generationId += 1
-      protocol = Some(selectProtocol)
-      transitionTo(CompletingRebalance)
+      protocol = selectProtocol
+      transitionTo(AwaitingSync)
     } else {
       generationId += 1
-      protocol = None
+      protocol = null
       transitionTo(Empty)
     }
     receivedConsumerOffsetCommits = false
@@ -288,20 +269,16 @@ private[group] class GroupMetadata(val groupId: String, initialState: GroupState
   def currentMemberMetadata: Map[String, Array[Byte]] = {
     if (is(Dead) || is(PreparingRebalance))
       throw new IllegalStateException("Cannot obtain member metadata for group in state %s".format(state))
-    members.map{ case (memberId, memberMetadata) => (memberId, memberMetadata.metadata(protocol.get))}.toMap
+    members.map{ case (memberId, memberMetadata) => (memberId, memberMetadata.metadata(protocol))}.toMap
   }
 
   def summary: GroupSummary = {
     if (is(Stable)) {
-      val protocol = protocolOrNull
-      if (protocol == null)
-        throw new IllegalStateException("Invalid null group protocol for stable group")
-
-      val members = this.members.values.map { member => member.summary(protocol) }
-      GroupSummary(state.toString, protocolType.getOrElse(""), protocol, members.toList)
+      val members = this.members.values.map { member => member.summary(protocol) }.toList
+      GroupSummary(state.toString, protocolType.getOrElse(""), protocol, members)
     } else {
-      val members = this.members.values.map{ member => member.summaryNoMetadata() }
-      GroupSummary(state.toString, protocolType.getOrElse(""), GroupCoordinator.NoProtocol, members.toList)
+      val members = this.members.values.map{ member => member.summaryNoMetadata() }.toList
+      GroupSummary(state.toString, protocolType.getOrElse(""), GroupCoordinator.NoProtocol, members)
     }
   }
 
@@ -398,7 +375,7 @@ private[group] class GroupMetadata(val groupId: String, initialState: GroupState
         pendingOffsets.foreach { case (topicPartition, commitRecordMetadataAndOffset) =>
           if (commitRecordMetadataAndOffset.appendedBatchOffset.isEmpty)
             throw new IllegalStateException(s"Trying to complete a transactional offset commit for producerId $producerId " +
-              s"and groupId $groupId even though the offset commit record itself hasn't been appended to the log.")
+              s"and groupId $groupId even though the the offset commit record itself hasn't been appended to the log.")
 
           val currentOffsetOpt = offsets.get(topicPartition)
           if (currentOffsetOpt.forall(_.olderThan(commitRecordMetadataAndOffset))) {
@@ -423,7 +400,6 @@ private[group] class GroupMetadata(val groupId: String, initialState: GroupState
 
   def removeOffsets(topicPartitions: Seq[TopicPartition]): immutable.Map[TopicPartition, OffsetAndMetadata] = {
     topicPartitions.flatMap { topicPartition =>
-
       pendingOffsetCommits.remove(topicPartition)
       pendingTransactionalOffsetCommits.foreach { case (_, pendingOffsets) =>
         pendingOffsets.remove(topicPartition)
