@@ -5,7 +5,7 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
@@ -19,6 +19,7 @@ package kafka.integration
 
 import java.io.File
 import java.util.Arrays
+
 import kafka.common.KafkaException
 import kafka.server._
 import kafka.utils.{CoreUtils, TestUtils}
@@ -26,13 +27,16 @@ import kafka.zk.ZooKeeperTestHarness
 import org.apache.kafka.common.protocol.SecurityProtocol
 import org.apache.kafka.common.security.auth.KafkaPrincipal
 import org.junit.{After, Before}
+
 import scala.collection.mutable.Buffer
 import java.util.Properties
 
+import org.apache.kafka.common.network.ListenerName
+
 /**
  * A test harness that brings up some number of broker nodes
  */
-trait KafkaServerTestHarness extends ZooKeeperTestHarness {
+abstract class KafkaServerTestHarness extends ZooKeeperTestHarness {
   var instanceConfigs: Seq[KafkaConfig] = null
   var servers: Buffer[KafkaServer] = null
   var brokerList: String = null
@@ -43,10 +47,10 @@ trait KafkaServerTestHarness extends ZooKeeperTestHarness {
    * Implementations must override this method to return a set of KafkaConfigs. This method will be invoked for every
    * test and should not reuse previous configurations unless they select their ports randomly when servers are started.
    */
-  def generateConfigs(): Seq[KafkaConfig]
+  def generateConfigs: Seq[KafkaConfig]
 
   /**
-   * Override this in case ACLs must be set before `servers` are started.
+   * Override this in case ACLs or security credentials must be set before `servers` are started.
    *
    * This is required in some cases because of the topic creation in the setup of `IntegrationTestHarness`. If the ACLs
    * are only set later, tests may fail. The failure could manifest itself as a cluster action
@@ -56,19 +60,23 @@ trait KafkaServerTestHarness extends ZooKeeperTestHarness {
    *
    * The default implementation of this method is a no-op.
    */
-  def setAclsBeforeServersStart() {}
+  def configureSecurityBeforeServersStart() {}
 
   def configs: Seq[KafkaConfig] = {
     if (instanceConfigs == null)
-      instanceConfigs = generateConfigs()
+      instanceConfigs = generateConfigs
     instanceConfigs
   }
 
   def serverForId(id: Int): Option[KafkaServer] = servers.find(s => s.config.brokerId == id)
 
+  def boundPort(server: KafkaServer): Int = server.boundPort(listenerName)
+
   protected def securityProtocol: SecurityProtocol = SecurityProtocol.PLAINTEXT
+  protected def listenerName: ListenerName = ListenerName.forSecurityProtocol(securityProtocol)
   protected def trustStoreFile: Option[File] = None
-  protected def saslProperties: Option[Properties] = None
+  protected def serverSaslProperties: Option[Properties] = None
+  protected def clientSaslProperties: Option[Properties] = None
 
   @Before
   override def setUp() {
@@ -78,10 +86,10 @@ trait KafkaServerTestHarness extends ZooKeeperTestHarness {
       throw new KafkaException("Must supply at least one server config.")
 
     // default implementation is a no-op, it is overridden by subclasses if required
-    setAclsBeforeServersStart()
+    configureSecurityBeforeServersStart()
 
     servers = configs.map(TestUtils.createServer(_)).toBuffer
-    brokerList = TestUtils.getBrokerListStrFromServers(servers, securityProtocol)
+    brokerList = TestUtils.bootstrapServers(servers, listenerName)
     alive = new Array[Boolean](servers.length)
     Arrays.fill(alive, true)
   }
@@ -89,31 +97,34 @@ trait KafkaServerTestHarness extends ZooKeeperTestHarness {
   @After
   override def tearDown() {
     if (servers != null) {
-      servers.foreach(_.shutdown())
-      servers.foreach(server => CoreUtils.delete(server.config.logDirs))
+      TestUtils.shutdownServers(servers)
     }
     super.tearDown
   }
-  
+
   /**
    * Pick a broker at random and kill it if it isn't already dead
    * Return the id of the broker killed
    */
   def killRandomBroker(): Int = {
     val index = TestUtils.random.nextInt(servers.length)
+    killBroker(index)
+    index
+  }
+
+  def killBroker(index: Int) {
     if(alive(index)) {
       servers(index).shutdown()
       servers(index).awaitShutdown()
       alive(index) = false
     }
-    index
   }
-  
+
   /**
    * Restart any dead brokers
    */
   def restartDeadBrokers() {
-    for(i <- 0 until servers.length if !alive(i)) {
+    for(i <- servers.indices if !alive(i)) {
       servers(i).startup()
       alive(i) = true
     }
