@@ -18,19 +18,14 @@
 package kafka.log
 
 import java.io.File
-import java.nio.ByteBuffer
-import java.nio.channels.FileChannel
-import java.nio.file.StandardOpenOption
-import java.util.Collections
 
 import kafka.server.LogOffsetMetadata
 import kafka.utils.TestUtils
 import org.apache.kafka.common.TopicPartition
 import org.apache.kafka.common.errors._
 import org.apache.kafka.common.internals.Topic
-import org.apache.kafka.common.record._
+import org.apache.kafka.common.record.{ControlRecordType, EndTransactionMarker, RecordBatch}
 import org.apache.kafka.common.utils.{MockTime, Utils}
-import org.easymock.EasyMock
 import org.junit.Assert._
 import org.junit.{After, Before, Test}
 import org.scalatest.junit.JUnitSuite
@@ -64,8 +59,8 @@ class ProducerStateManagerTest extends JUnitSuite {
     // Second entry for id 0 added
     append(stateManager, producerId, epoch, 1, 0L, 1L)
 
-    // Duplicates are checked separately and should result in OutOfOrderSequence if appended
-    assertThrows[OutOfOrderSequenceException] {
+    // Duplicate sequence number (matches previous sequence number)
+    assertThrows[DuplicateSequenceNumberException] {
       append(stateManager, producerId, epoch, 1, 0L, 1L)
     }
 
@@ -83,41 +78,12 @@ class ProducerStateManagerTest extends JUnitSuite {
     }
   }
 
-  @Test
-  def testAppendTxnMarkerWithNoProducerState(): Unit = {
-    val producerEpoch = 2.toShort
-    appendEndTxnMarker(stateManager, producerId, producerEpoch, ControlRecordType.COMMIT, offset = 27L)
-
-    val firstEntry = stateManager.lastEntry(producerId).getOrElse(fail("Expected last entry to be defined"))
-    assertEquals(producerEpoch, firstEntry.producerEpoch)
-    assertEquals(producerId, firstEntry.producerId)
-    assertEquals(RecordBatch.NO_SEQUENCE, firstEntry.lastSeq)
-
-    // Fencing should continue to work even if the marker is the only thing left
-    assertThrows[ProducerFencedException] {
-      append(stateManager, producerId, 0.toShort, 0, 0L, 4L)
-    }
-
-    // If the transaction marker is the only thing left in the log, then an attempt to write using a
-    // non-zero sequence number should cause an UnknownProducerId, so that the producer can reset its state
-    assertThrows[UnknownProducerIdException] {
-      append(stateManager, producerId, producerEpoch, 17, 0L, 4L)
-    }
-
-    // The broker should accept the request if the sequence number is reset to 0
-    append(stateManager, producerId, producerEpoch, 0, 39L, 4L)
-    val secondEntry = stateManager.lastEntry(producerId).getOrElse(fail("Expected last entry to be defined"))
-    assertEquals(producerEpoch, secondEntry.producerEpoch)
-    assertEquals(producerId, secondEntry.producerId)
-    assertEquals(0, secondEntry.lastSeq)
-  }
-
   @Test
   def testProducerSequenceWrapAround(): Unit = {
     val epoch = 15.toShort
     val sequence = Int.MaxValue
     val offset = 735L
-    append(stateManager, producerId, epoch, sequence, offset, isFromClient = false)
+    append(stateManager, producerId, epoch, sequence, offset, isLoadingFromLog = true)
 
     append(stateManager, producerId, epoch, 0, offset + 500)
 
@@ -126,8 +92,7 @@ class ProducerStateManagerTest extends JUnitSuite {
 
     val lastEntry = maybeLastEntry.get
     assertEquals(epoch, lastEntry.producerEpoch)
-
-    assertEquals(Int.MaxValue, lastEntry.firstSeq)
+    assertEquals(0, lastEntry.firstSeq)
     assertEquals(0, lastEntry.lastSeq)
   }
 
@@ -136,7 +101,7 @@ class ProducerStateManagerTest extends JUnitSuite {
     val epoch = 15.toShort
     val sequence = Int.MaxValue
     val offset = 735L
-    append(stateManager, producerId, epoch, sequence, offset, isFromClient = false)
+    append(stateManager, producerId, epoch, sequence, offset, isLoadingFromLog = true)
     append(stateManager, producerId, epoch, 1, offset + 500)
   }
 
@@ -145,7 +110,7 @@ class ProducerStateManagerTest extends JUnitSuite {
     val epoch = 5.toShort
     val sequence = 16
     val offset = 735L
-    append(stateManager, producerId, epoch, sequence, offset, isFromClient = false)
+    append(stateManager, producerId, epoch, sequence, offset, isLoadingFromLog = true)
 
     val maybeLastEntry = stateManager.lastEntry(producerId)
     assertTrue(maybeLastEntry.isDefined)
@@ -154,7 +119,7 @@ class ProducerStateManagerTest extends JUnitSuite {
     assertEquals(epoch, lastEntry.producerEpoch)
     assertEquals(sequence, lastEntry.firstSeq)
     assertEquals(sequence, lastEntry.lastSeq)
-    assertEquals(offset, lastEntry.lastDataOffset)
+    assertEquals(offset, lastEntry.lastOffset)
     assertEquals(offset, lastEntry.firstOffset)
   }
 
@@ -190,7 +155,8 @@ class ProducerStateManagerTest extends JUnitSuite {
     val producerEpoch = 0.toShort
     val offset = 992342L
     val seq = 0
-    val producerAppendInfo = new ProducerAppendInfo(producerId, ProducerStateEntry.empty(producerId), ValidationType.Full)
+    val producerAppendInfo = new ProducerAppendInfo(producerId, ProducerIdEntry.Empty, validateSequenceNumbers = true,
+      loadingFromLog = false)
     producerAppendInfo.append(producerEpoch, seq, seq, time.milliseconds(), offset, isTransactional = true)
 
     val logOffsetMetadata = new LogOffsetMetadata(messageOffset = offset, segmentBaseOffset = 990000L,
@@ -206,7 +172,8 @@ class ProducerStateManagerTest extends JUnitSuite {
     val producerEpoch = 0.toShort
     val offset = 992342L
     val seq = 0
-    val producerAppendInfo = new ProducerAppendInfo(producerId, ProducerStateEntry.empty(producerId), ValidationType.Full)
+    val producerAppendInfo = new ProducerAppendInfo(producerId, ProducerIdEntry.Empty, validateSequenceNumbers = true,
+      loadingFromLog = false)
     producerAppendInfo.append(producerEpoch, seq, seq, time.milliseconds(), offset, isTransactional = true)
 
     // use some other offset to simulate a follower append where the log offset metadata won't typically
@@ -219,32 +186,6 @@ class ProducerStateManagerTest extends JUnitSuite {
     assertEquals(Some(LogOffsetMetadata(offset)), stateManager.firstUnstableOffset)
   }
 
-  @Test
-  def testPrepareUpdateDoesNotMutate(): Unit = {
-    val producerEpoch = 0.toShort
-
-    val appendInfo = stateManager.prepareUpdate(producerId, isFromClient = true)
-    appendInfo.append(producerEpoch, 0, 5, time.milliseconds(), 20L, isTransactional = false)
-    assertEquals(None, stateManager.lastEntry(producerId))
-    stateManager.update(appendInfo)
-    assertTrue(stateManager.lastEntry(producerId).isDefined)
-
-    val nextAppendInfo = stateManager.prepareUpdate(producerId, isFromClient = true)
-    nextAppendInfo.append(producerEpoch, 6, 10, time.milliseconds(), 30L, isTransactional = false)
-    assertTrue(stateManager.lastEntry(producerId).isDefined)
-
-    var lastEntry = stateManager.lastEntry(producerId).get
-    assertEquals(0, lastEntry.firstSeq)
-    assertEquals(5, lastEntry.lastSeq)
-    assertEquals(20L, lastEntry.lastDataOffset)
-
-    stateManager.update(nextAppendInfo)
-    lastEntry = stateManager.lastEntry(producerId).get
-    assertEquals(0, lastEntry.firstSeq)
-    assertEquals(10, lastEntry.lastSeq)
-    assertEquals(30L, lastEntry.lastDataOffset)
-  }
-
   @Test
   def updateProducerTransactionState(): Unit = {
     val producerEpoch = 0.toShort
@@ -252,24 +193,24 @@ class ProducerStateManagerTest extends JUnitSuite {
     val offset = 9L
     append(stateManager, producerId, producerEpoch, 0, offset)
 
-    val appendInfo = stateManager.prepareUpdate(producerId, isFromClient = true)
+    val appendInfo = stateManager.prepareUpdate(producerId, loadingFromLog = false)
     appendInfo.append(producerEpoch, 1, 5, time.milliseconds(), 20L, isTransactional = true)
-    var lastEntry = appendInfo.toEntry
+    var lastEntry = appendInfo.lastEntry
     assertEquals(producerEpoch, lastEntry.producerEpoch)
     assertEquals(1, lastEntry.firstSeq)
     assertEquals(5, lastEntry.lastSeq)
     assertEquals(16L, lastEntry.firstOffset)
-    assertEquals(20L, lastEntry.lastDataOffset)
+    assertEquals(20L, lastEntry.lastOffset)
     assertEquals(Some(16L), lastEntry.currentTxnFirstOffset)
     assertEquals(List(new TxnMetadata(producerId, 16L)), appendInfo.startedTransactions)
 
     appendInfo.append(producerEpoch, 6, 10, time.milliseconds(), 30L, isTransactional = true)
-    lastEntry = appendInfo.toEntry
+    lastEntry = appendInfo.lastEntry
     assertEquals(producerEpoch, lastEntry.producerEpoch)
-    assertEquals(1, lastEntry.firstSeq)
+    assertEquals(6, lastEntry.firstSeq)
     assertEquals(10, lastEntry.lastSeq)
-    assertEquals(16L, lastEntry.firstOffset)
-    assertEquals(30L, lastEntry.lastDataOffset)
+    assertEquals(26L, lastEntry.firstOffset)
+    assertEquals(30L, lastEntry.lastOffset)
     assertEquals(Some(16L), lastEntry.currentTxnFirstOffset)
     assertEquals(List(new TxnMetadata(producerId, 16L)), appendInfo.startedTransactions)
 
@@ -280,40 +221,28 @@ class ProducerStateManagerTest extends JUnitSuite {
     assertEquals(40L, completedTxn.lastOffset)
     assertFalse(completedTxn.isAborted)
 
-    lastEntry = appendInfo.toEntry
+    lastEntry = appendInfo.lastEntry
     assertEquals(producerEpoch, lastEntry.producerEpoch)
-    // verify that appending the transaction marker doesn't affect the metadata of the cached record batches.
-    assertEquals(1, lastEntry.firstSeq)
+    assertEquals(10, lastEntry.firstSeq)
     assertEquals(10, lastEntry.lastSeq)
-    assertEquals(16L, lastEntry.firstOffset)
-    assertEquals(30L, lastEntry.lastDataOffset)
+    assertEquals(40L, lastEntry.firstOffset)
+    assertEquals(40L, lastEntry.lastOffset)
     assertEquals(coordinatorEpoch, lastEntry.coordinatorEpoch)
     assertEquals(None, lastEntry.currentTxnFirstOffset)
     assertEquals(List(new TxnMetadata(producerId, 16L)), appendInfo.startedTransactions)
   }
 
-  @Test
+  @Test(expected = classOf[OutOfOrderSequenceException])
   def testOutOfSequenceAfterControlRecordEpochBump(): Unit = {
     val epoch = 0.toShort
-    append(stateManager, producerId, epoch, 0, 0L, isTransactional = true)
-    append(stateManager, producerId, epoch, 1, 1L, isTransactional = true)
+    append(stateManager, producerId, epoch, 0, 0L)
+    append(stateManager, producerId, epoch, 1, 1L)
 
     val bumpedEpoch = 1.toShort
     appendEndTxnMarker(stateManager, producerId, bumpedEpoch, ControlRecordType.ABORT, 1L)
 
     // next append is invalid since we expect the sequence to be reset
-    assertThrows[OutOfOrderSequenceException] {
-      append(stateManager, producerId, bumpedEpoch, 2, 2L, isTransactional = true)
-    }
-
-    assertThrows[OutOfOrderSequenceException] {
-      append(stateManager, producerId, (bumpedEpoch + 1).toShort, 2, 2L, isTransactional = true)
-    }
-
-    // Append with the bumped epoch should be fine if starting from sequence 0
-    append(stateManager, producerId, bumpedEpoch, 0, 0L, isTransactional = true)
-    assertEquals(bumpedEpoch, stateManager.lastEntry(producerId).get.producerEpoch)
-    assertEquals(0, stateManager.lastEntry(producerId).get.lastSeq)
+    append(stateManager, producerId, bumpedEpoch, 2, 2L)
   }
 
   @Test(expected = classOf[InvalidTxnStateException])
@@ -371,7 +300,7 @@ class ProducerStateManagerTest extends JUnitSuite {
     append(recoveredMapping, producerId, epoch, 2, 2L)
   }
 
-  @Test(expected = classOf[UnknownProducerIdException])
+  @Test(expected = classOf[OutOfOrderSequenceException])
   def testRemoveExpiredPidsOnReload(): Unit = {
     val epoch = 0.toShort
     append(stateManager, producerId, epoch, 0, 0L, 0)
@@ -386,50 +315,6 @@ class ProducerStateManagerTest extends JUnitSuite {
     append(recoveredMapping, producerId, epoch, 2, 2L, 70001)
   }
 
-  @Test
-  def testAcceptAppendWithoutProducerStateOnReplica(): Unit = {
-    val epoch = 0.toShort
-    append(stateManager, producerId, epoch, 0, 0L, 0)
-    append(stateManager, producerId, epoch, 1, 1L, 1)
-
-    stateManager.takeSnapshot()
-    val recoveredMapping = new ProducerStateManager(partition, logDir, maxPidExpirationMs)
-    recoveredMapping.truncateAndReload(0L, 1L, 70000)
-
-    val sequence = 2
-    // entry added after recovery. The pid should be expired now, and would not exist in the pid mapping. Nonetheless
-    // the append on a replica should be accepted with the local producer state updated to the appended value.
-    assertFalse(recoveredMapping.activeProducers.contains(producerId))
-    append(recoveredMapping, producerId, epoch, sequence, 2L, 70001, isFromClient = false)
-    assertTrue(recoveredMapping.activeProducers.contains(producerId))
-    val producerStateEntry = recoveredMapping.activeProducers.get(producerId).head
-    assertEquals(epoch, producerStateEntry.producerEpoch)
-    assertEquals(sequence, producerStateEntry.firstSeq)
-    assertEquals(sequence, producerStateEntry.lastSeq)
-  }
-
-  @Test
-  def testAcceptAppendWithSequenceGapsOnReplica(): Unit = {
-    val epoch = 0.toShort
-    append(stateManager, producerId, epoch, 0, 0L, 0)
-    val outOfOrderSequence = 3
-
-    // First we ensure that we raise an OutOfOrderSequenceException is raised when the append comes from a client.
-    try {
-      append(stateManager, producerId, epoch, outOfOrderSequence, 1L, 1, isFromClient = true)
-      fail("Expected an OutOfOrderSequenceException to be raised.")
-    } catch {
-      case _ : OutOfOrderSequenceException =>
-      // Good!
-      case _ : Exception =>
-        fail("Expected an OutOfOrderSequenceException to be raised.")
-    }
-
-    assertEquals(0L, stateManager.activeProducers(producerId).lastSeq)
-    append(stateManager, producerId, epoch, outOfOrderSequence, 1L, 1, isFromClient = false)
-    assertEquals(outOfOrderSequence, stateManager.activeProducers(producerId).lastSeq)
-  }
-
   @Test
   def testDeleteSnapshotsBefore(): Unit = {
     val epoch = 0.toShort
@@ -533,7 +418,7 @@ class ProducerStateManagerTest extends JUnitSuite {
 
     val maybeEntry = stateManager.lastEntry(anotherPid)
     assertTrue(maybeEntry.isDefined)
-    assertEquals(3L, maybeEntry.get.lastDataOffset)
+    assertEquals(3L, maybeEntry.get.lastOffset)
 
     stateManager.truncateHead(3)
     assertEquals(Set(anotherPid), stateManager.activeProducers.keySet)
@@ -564,7 +449,7 @@ class ProducerStateManagerTest extends JUnitSuite {
     val entry = stateManager.lastEntry(pid2)
     assertTrue(entry.isDefined)
     assertEquals(0, entry.get.lastSeq)
-    assertEquals(1L, entry.get.lastDataOffset)
+    assertEquals(1L, entry.get.lastOffset)
   }
 
   @Test
@@ -592,14 +477,14 @@ class ProducerStateManagerTest extends JUnitSuite {
     append(stateManager, producerId, epoch, 2, 3L, 4L)
     stateManager.takeSnapshot()
 
-    assertThrows[UnknownProducerIdException] {
+    intercept[OutOfOrderSequenceException] {
       val recoveredMapping = new ProducerStateManager(partition, logDir, maxPidExpirationMs)
       recoveredMapping.truncateAndReload(0L, 1L, time.milliseconds)
       append(recoveredMapping, pid2, epoch, 1, 4L, 5L)
     }
   }
 
-  @Test(expected = classOf[UnknownProducerIdException])
+  @Test(expected = classOf[OutOfOrderSequenceException])
   def testPidExpirationTimeout() {
     val epoch = 5.toShort
     val sequence = 37
@@ -723,77 +608,6 @@ class ProducerStateManagerTest extends JUnitSuite {
     appendEndTxnMarker(stateManager, producerId, producerEpoch, ControlRecordType.COMMIT, offset = 100, coordinatorEpoch = 0)
   }
 
-  @Test
-  def testLoadFromEmptySnapshotFile(): Unit = {
-    testLoadFromCorruptSnapshot { file =>
-      file.truncate(0L)
-    }
-  }
-
-  @Test
-  def testLoadFromTruncatedSnapshotFile(): Unit = {
-    testLoadFromCorruptSnapshot { file =>
-      // truncate to some arbitrary point in the middle of the snapshot
-      assertTrue(file.size > 2)
-      file.truncate(file.size / 2)
-    }
-  }
-
-  @Test
-  def testLoadFromCorruptSnapshotFile(): Unit = {
-    testLoadFromCorruptSnapshot { file =>
-      // write some garbage somewhere in the file
-      assertTrue(file.size > 2)
-      file.write(ByteBuffer.wrap(Array[Byte](37)), file.size / 2)
-    }
-  }
-
-  @Test
-  def testAppendEmptyControlBatch(): Unit = {
-    val producerId = 23423L
-    val producerEpoch = 145.toShort
-    val baseOffset = 15
-
-    val batch = EasyMock.createMock(classOf[RecordBatch])
-    EasyMock.expect(batch.isControlBatch).andReturn(true).once
-    EasyMock.expect(batch.iterator).andReturn(Collections.emptyIterator[Record]).once
-    EasyMock.replay(batch)
-
-    // Appending the empty control batch should not throw and a new transaction shouldn't be started
-    append(stateManager, producerId, producerEpoch, baseOffset, batch, isFromClient = true)
-    assertEquals(None, stateManager.lastEntry(producerId).get.currentTxnFirstOffset)
-  }
-
-  private def testLoadFromCorruptSnapshot(makeFileCorrupt: FileChannel => Unit): Unit = {
-    val epoch = 0.toShort
-    val producerId = 1L
-
-    append(stateManager, producerId, epoch, seq = 0, offset = 0L)
-    stateManager.takeSnapshot()
-
-    append(stateManager, producerId, epoch, seq = 1, offset = 1L)
-    stateManager.takeSnapshot()
-
-    // Truncate the last snapshot
-    val latestSnapshotOffset = stateManager.latestSnapshotOffset
-    assertEquals(Some(2L), latestSnapshotOffset)
-    val snapshotToTruncate = Log.producerSnapshotFile(logDir, latestSnapshotOffset.get)
-    val channel = FileChannel.open(snapshotToTruncate.toPath, StandardOpenOption.WRITE)
-    try {
-      makeFileCorrupt(channel)
-    } finally {
-      channel.close()
-    }
-
-    // Ensure that the truncated snapshot is deleted and producer state is loaded from the previous snapshot
-    val reloadedStateManager = new ProducerStateManager(partition, logDir, maxPidExpirationMs)
-    reloadedStateManager.truncateAndReload(0L, 20L, time.milliseconds())
-    assertFalse(snapshotToTruncate.exists())
-
-    val loadedProducerState = reloadedStateManager.activeProducers(producerId)
-    assertEquals(0L, loadedProducerState.lastDataOffset)
-  }
-
   private def appendEndTxnMarker(mapping: ProducerStateManager,
                                  producerId: Long,
                                  producerEpoch: Short,
@@ -801,7 +615,7 @@ class ProducerStateManagerTest extends JUnitSuite {
                                  offset: Long,
                                  coordinatorEpoch: Int = 0,
                                  timestamp: Long = time.milliseconds()): (CompletedTxn, Long) = {
-    val producerAppendInfo = stateManager.prepareUpdate(producerId, isFromClient = true)
+    val producerAppendInfo = stateManager.prepareUpdate(producerId, loadingFromLog = false)
     val endTxnMarker = new EndTransactionMarker(controlType, coordinatorEpoch)
     val completedTxn = producerAppendInfo.appendEndTxnMarker(endTxnMarker, producerEpoch, offset, timestamp)
     mapping.update(producerAppendInfo)
@@ -817,26 +631,14 @@ class ProducerStateManagerTest extends JUnitSuite {
                      offset: Long,
                      timestamp: Long = time.milliseconds(),
                      isTransactional: Boolean = false,
-                     isFromClient : Boolean = true): Unit = {
-    val producerAppendInfo = stateManager.prepareUpdate(producerId, isFromClient)
+                     isLoadingFromLog: Boolean = false): Unit = {
+    val producerAppendInfo = stateManager.prepareUpdate(producerId, isLoadingFromLog)
     producerAppendInfo.append(producerEpoch, seq, seq, timestamp, offset, isTransactional)
     stateManager.update(producerAppendInfo)
     stateManager.updateMapEndOffset(offset + 1)
   }
 
-  private def append(stateManager: ProducerStateManager,
-                     producerId: Long,
-                     producerEpoch: Short,
-                     offset: Long,
-                     batch: RecordBatch,
-                     isFromClient : Boolean): Unit = {
-    val producerAppendInfo = stateManager.prepareUpdate(producerId, isFromClient)
-    producerAppendInfo.append(batch)
-    stateManager.update(producerAppendInfo)
-    stateManager.updateMapEndOffset(offset + 1)
-  }
-
   private def currentSnapshotOffsets =
-    logDir.listFiles.map(Log.offsetFromFile).toSet
+    logDir.listFiles().map(file => Log.offsetFromFilename(file.getName)).toSet
 
 }
