@@ -16,19 +16,19 @@
   */
 package kafka.server
 
-import java.io.DataInputStream
 import java.util
 import java.util.Properties
 
 import kafka.api.KAFKA_0_11_0_IV2
 import kafka.log.LogConfig
 import kafka.utils.TestUtils
-import org.apache.kafka.clients.producer.{KafkaProducer, ProducerConfig, ProducerRecord}
+import kafka.utils.TestUtils._
+import org.apache.kafka.clients.producer.{KafkaProducer, ProducerRecord}
 import org.apache.kafka.common.TopicPartition
 import org.apache.kafka.common.protocol.{ApiKeys, Errors}
-import org.apache.kafka.common.record.{MemoryRecords, Record, RecordBatch}
-import org.apache.kafka.common.requests.{FetchRequest, FetchResponse, IsolationLevel, FetchMetadata => JFetchMetadata}
-import org.apache.kafka.common.serialization.{ByteArraySerializer, StringSerializer}
+import org.apache.kafka.common.record.{Record, RecordBatch}
+import org.apache.kafka.common.requests.{FetchRequest, FetchResponse, IsolationLevel}
+import org.apache.kafka.common.serialization.StringSerializer
 import org.junit.Assert._
 import org.junit.Test
 
@@ -44,13 +44,12 @@ class FetchRequestTest extends BaseRequestTest {
   private var producer: KafkaProducer[String, String] = null
 
   override def tearDown() {
-    if (producer != null)
-      producer.close()
+    producer.close()
     super.tearDown()
   }
 
   private def createFetchRequest(maxResponseBytes: Int, maxPartitionBytes: Int, topicPartitions: Seq[TopicPartition],
-                                 offsetMap: Map[TopicPartition, Long]): FetchRequest =
+                                 offsetMap: Map[TopicPartition, Long] = Map.empty): FetchRequest =
     FetchRequest.Builder.forConsumer(Int.MaxValue, 0, createPartitionMap(maxPartitionBytes, topicPartitions, offsetMap))
       .setMaxBytes(maxResponseBytes).build()
 
@@ -63,13 +62,13 @@ class FetchRequestTest extends BaseRequestTest {
     partitionMap
   }
 
-  private def sendFetchRequest(leaderId: Int, request: FetchRequest): FetchResponse[MemoryRecords] = {
+  private def sendFetchRequest(leaderId: Int, request: FetchRequest): FetchResponse = {
     val response = connectAndSend(request, ApiKeys.FETCH, destination = brokerSocketServer(leaderId))
     FetchResponse.parse(response, request.version)
   }
 
   private def initProducer(): Unit = {
-    producer = TestUtils.createProducer(TestUtils.getBrokerListStrFromServers(servers),
+    producer = TestUtils.createNewProducer(TestUtils.getBrokerListStrFromServers(servers),
       retries = 5, keySerializer = new StringSerializer, valueSerializer = new StringSerializer)
   }
 
@@ -169,106 +168,6 @@ class FetchRequestTest extends BaseRequestTest {
     assertEquals(0, records(partitionData).map(_.sizeInBytes).sum)
   }
 
-  @Test
-  def testFetchRequestV4WithReadCommitted(): Unit = {
-    initProducer()
-    val maxPartitionBytes = 200
-    val (topicPartition, leaderId) = createTopics(numTopics = 1, numPartitions = 1).head
-    producer.send(new ProducerRecord(topicPartition.topic, topicPartition.partition,
-      "key", new String(new Array[Byte](maxPartitionBytes + 1)))).get
-    val fetchRequest = FetchRequest.Builder.forConsumer(Int.MaxValue, 0, createPartitionMap(maxPartitionBytes,
-      Seq(topicPartition))).isolationLevel(IsolationLevel.READ_COMMITTED).build(4)
-    val fetchResponse = sendFetchRequest(leaderId, fetchRequest)
-    val partitionData = fetchResponse.responseData.get(topicPartition)
-    assertEquals(Errors.NONE, partitionData.error)
-    assertTrue(partitionData.lastStableOffset > 0)
-    assertTrue(records(partitionData).map(_.sizeInBytes).sum > 0)
-  }
-
-  @Test
-  def testFetchRequestToNonReplica(): Unit = {
-    val topic = "topic"
-    val partition = 0
-    val topicPartition = new TopicPartition(topic, partition)
-
-    // Create a single-partition topic and find a broker which is not the leader
-    val partitionToLeader = TestUtils.createTopic(zkClient, topic, numPartitions = 1, 1, servers)
-    val leader = partitionToLeader(partition)
-    val nonReplicaOpt = servers.find(_.config.brokerId != leader)
-    assertTrue(nonReplicaOpt.isDefined)
-    val nonReplicaId =  nonReplicaOpt.get.config.brokerId
-
-    // Send the fetch request to the non-replica and verify the error code
-    val fetchRequest = FetchRequest.Builder.forConsumer(Int.MaxValue, 0, createPartitionMap(1024,
-      Seq(topicPartition))).build()
-    val fetchResponse = sendFetchRequest(nonReplicaId, fetchRequest)
-    val partitionData = fetchResponse.responseData.get(topicPartition)
-    assertEquals(Errors.NOT_LEADER_FOR_PARTITION, partitionData.error)
-  }
-
-  /**
-   * Tests that down-conversions dont leak memory. Large down conversions are triggered
-   * in the server. The client closes its connection after reading partial data when the
-   * channel is muted in the server. If buffers are not released this will result in OOM.
-   */
-  @Test
-  def testDownConversionWithConnectionFailure(): Unit = {
-    val (topicPartition, leaderId) = createTopics(numTopics = 1, numPartitions = 1).head
-
-    val msgValueLen = 100 * 1000
-    val batchSize = 4 * msgValueLen
-    val propsOverride = new Properties
-    propsOverride.put(ProducerConfig.BATCH_SIZE_CONFIG, batchSize.toString)
-    val producer = TestUtils.createProducer(TestUtils.getBrokerListStrFromServers(servers),
-      retries = 5, lingerMs = Long.MaxValue,
-      keySerializer = new StringSerializer, valueSerializer = new ByteArraySerializer, props = Some(propsOverride))
-    val bytes = new Array[Byte](msgValueLen)
-    val futures = try {
-      (0 to 1000).map { _ =>
-        producer.send(new ProducerRecord(topicPartition.topic, topicPartition.partition, "key", bytes))
-      }
-    } finally {
-      producer.close()
-    }
-    // Check futures to ensure sends succeeded, but do this after close since the last
-    // batch is not complete, but sent when the producer is closed
-    futures.foreach(_.get)
-
-    def fetch(version: Short, maxPartitionBytes: Int, closeAfterPartialResponse: Boolean): Option[FetchResponse[MemoryRecords]] = {
-      val fetchRequest = FetchRequest.Builder.forConsumer(Int.MaxValue, 0, createPartitionMap(maxPartitionBytes,
-        Seq(topicPartition))).build(version)
-
-      val socket = connect(brokerSocketServer(leaderId))
-      try {
-        send(fetchRequest, ApiKeys.FETCH, socket)
-        if (closeAfterPartialResponse) {
-          // read some data to ensure broker has muted this channel and then close socket
-          val size = new DataInputStream(socket.getInputStream).readInt()
-          // Check that we have received almost `maxPartitionBytes` (minus a tolerance) since in
-          // the case of OOM, the size will be significantly smaller. We can't check for exactly
-          // maxPartitionBytes since we use approx message sizes that include only the message value.
-          assertTrue(s"Fetch size too small $size, broker may have run out of memory",
-              size > maxPartitionBytes - batchSize)
-          None
-        } else {
-          Some(FetchResponse.parse(receive(socket), version))
-        }
-      } finally {
-        socket.close()
-      }
-    }
-
-    val version = 1.toShort
-    (0 to 15).foreach(_ => fetch(version, maxPartitionBytes = msgValueLen * 1000, closeAfterPartialResponse = true))
-
-    val response = fetch(version, maxPartitionBytes = batchSize, closeAfterPartialResponse = false)
-    val fetchResponse = response.getOrElse(throw new IllegalStateException("No fetch response"))
-    val partitionData = fetchResponse.responseData.get(topicPartition)
-    assertEquals(Errors.NONE, partitionData.error)
-    val batches = partitionData.records.batches.asScala.toBuffer
-    assertEquals(3, batches.size) // size is 3 (not 4) since maxPartitionBytes=msgValueSize*4, excluding key and headers
-  }
-
   /**
     * Ensure that we respect the fetch offset when returning records that were converted from an uncompressed v2
     * record batch to multiple v0/v1 record batches with size 1. If the fetch offset points to inside the record batch,
@@ -277,7 +176,7 @@ class FetchRequestTest extends BaseRequestTest {
   @Test
   def testDownConversionFromBatchedToUnbatchedRespectsOffset(): Unit = {
     // Increase linger so that we have control over the batches created
-    producer = TestUtils.createProducer(TestUtils.getBrokerListStrFromServers(servers),
+    producer = TestUtils.createNewProducer(TestUtils.getBrokerListStrFromServers(servers),
       retries = 5, keySerializer = new StringSerializer, valueSerializer = new StringSerializer,
       lingerMs = 300 * 1000)
 
@@ -294,32 +193,17 @@ class FetchRequestTest extends BaseRequestTest {
     secondBatchFutures.foreach(_.get)
 
     def check(fetchOffset: Long, requestVersion: Short, expectedOffset: Long, expectedNumBatches: Int, expectedMagic: Byte): Unit = {
-      var batchesReceived = 0
-      var currentFetchOffset = fetchOffset
-      var currentExpectedOffset = expectedOffset
-
-      // With KIP-283, we might not receive all batches in a single fetch request so loop through till we have consumed
-      // all batches we are interested in.
-      while (batchesReceived < expectedNumBatches) {
-        val fetchRequest = FetchRequest.Builder.forConsumer(Int.MaxValue, 0, createPartitionMap(Int.MaxValue,
-          Seq(topicPartition), Map(topicPartition -> currentFetchOffset))).build(requestVersion)
-        val fetchResponse = sendFetchRequest(leaderId, fetchRequest)
-
-        // validate response
-        val partitionData = fetchResponse.responseData.get(topicPartition)
-        assertEquals(Errors.NONE, partitionData.error)
-        assertTrue(partitionData.highWatermark > 0)
-        val batches = partitionData.records.batches.asScala.toBuffer
-        val batch = batches.head
-        assertEquals(expectedMagic, batch.magic)
-        assertEquals(currentExpectedOffset, batch.baseOffset)
-
-        currentFetchOffset = batches.last.lastOffset + 1
-        currentExpectedOffset += (batches.last.lastOffset - batches.head.baseOffset + 1)
-        batchesReceived += batches.size
-      }
-
-      assertEquals(expectedNumBatches, batchesReceived)
+      val fetchRequest = FetchRequest.Builder.forConsumer(Int.MaxValue, 0, createPartitionMap(Int.MaxValue,
+        Seq(topicPartition), Map(topicPartition -> fetchOffset))).build(requestVersion)
+      val fetchResponse = sendFetchRequest(leaderId, fetchRequest)
+      val partitionData = fetchResponse.responseData.get(topicPartition)
+      assertEquals(Errors.NONE, partitionData.error)
+      assertTrue(partitionData.highWatermark > 0)
+      val batches = partitionData.records.batches.asScala.toBuffer
+      assertEquals(expectedNumBatches, batches.size)
+      val batch = batches.head
+      assertEquals(expectedMagic, batch.magic)
+      assertEquals(expectedOffset, batch.baseOffset)
     }
 
     // down conversion to message format 0, batches of 1 message are returned so we receive the exact offset we requested
@@ -346,65 +230,11 @@ class FetchRequestTest extends BaseRequestTest {
       expectedMagic = RecordBatch.MAGIC_VALUE_V2)
   }
 
-  /**
-   * Test that when an incremental fetch session contains partitions with an error,
-   * those partitions are returned in all incremental fetch requests.
-   */
-  @Test
-  def testCreateIncrementalFetchWithPartitionsInError(): Unit = {
-    def createFetchRequest(topicPartitions: Seq[TopicPartition],
-                           metadata: JFetchMetadata,
-                           toForget: Seq[TopicPartition]): FetchRequest =
-      FetchRequest.Builder.forConsumer(Int.MaxValue, 0,
-        createPartitionMap(Integer.MAX_VALUE, topicPartitions, Map.empty))
-        .toForget(toForget.asJava)
-        .metadata(metadata)
-        .build()
-    val foo0 = new TopicPartition("foo", 0)
-    val foo1 = new TopicPartition("foo", 1)
-    createTopic("foo", Map(0 -> List(0, 1), 1 -> List(0, 2)))
-    val bar0 = new TopicPartition("bar", 0)
-    val req1 = createFetchRequest(List(foo0, foo1, bar0), JFetchMetadata.INITIAL, Nil)
-    val resp1 = sendFetchRequest(0, req1)
-    assertEquals(Errors.NONE, resp1.error())
-    assertTrue("Expected the broker to create a new incremental fetch session", resp1.sessionId() > 0)
-    debug(s"Test created an incremental fetch session ${resp1.sessionId}")
-    assertTrue(resp1.responseData().containsKey(foo0))
-    assertTrue(resp1.responseData().containsKey(foo1))
-    assertTrue(resp1.responseData().containsKey(bar0))
-    assertEquals(Errors.NONE, resp1.responseData().get(foo0).error)
-    assertEquals(Errors.NONE, resp1.responseData().get(foo1).error)
-    assertEquals(Errors.UNKNOWN_TOPIC_OR_PARTITION, resp1.responseData().get(bar0).error)
-    val req2 = createFetchRequest(Nil, new JFetchMetadata(resp1.sessionId(), 1), Nil)
-    val resp2 = sendFetchRequest(0, req2)
-    assertEquals(Errors.NONE, resp2.error())
-    assertEquals("Expected the broker to continue the incremental fetch session",
-      resp1.sessionId(), resp2.sessionId())
-    assertFalse(resp2.responseData().containsKey(foo0))
-    assertFalse(resp2.responseData().containsKey(foo1))
-    assertTrue(resp2.responseData().containsKey(bar0))
-    assertEquals(Errors.UNKNOWN_TOPIC_OR_PARTITION, resp2.responseData().get(bar0).error)
-    createTopic("bar", Map(0 -> List(0, 1)))
-    val req3 = createFetchRequest(Nil, new JFetchMetadata(resp1.sessionId(), 2), Nil)
-    val resp3 = sendFetchRequest(0, req3)
-    assertEquals(Errors.NONE, resp3.error())
-    assertFalse(resp3.responseData().containsKey(foo0))
-    assertFalse(resp3.responseData().containsKey(foo1))
-    assertTrue(resp3.responseData().containsKey(bar0))
-    assertEquals(Errors.NONE, resp3.responseData().get(bar0).error)
-    val req4 = createFetchRequest(Nil, new JFetchMetadata(resp1.sessionId(), 3), Nil)
-    val resp4 = sendFetchRequest(0, req4)
-    assertEquals(Errors.NONE, resp4.error())
-    assertFalse(resp4.responseData().containsKey(foo0))
-    assertFalse(resp4.responseData().containsKey(foo1))
-    assertFalse(resp4.responseData().containsKey(bar0))
-  }
-
-  private def records(partitionData: FetchResponse.PartitionData[MemoryRecords]): Seq[Record] = {
+  private def records(partitionData: FetchResponse.PartitionData): Seq[Record] = {
     partitionData.records.records.asScala.toIndexedSeq
   }
 
-  private def checkFetchResponse(expectedPartitions: Seq[TopicPartition], fetchResponse: FetchResponse[MemoryRecords],
+  private def checkFetchResponse(expectedPartitions: Seq[TopicPartition], fetchResponse: FetchResponse,
                                  maxPartitionBytes: Int, maxResponseBytes: Int, numMessagesPerPartition: Int): Unit = {
     assertEquals(expectedPartitions, fetchResponse.responseData.keySet.asScala.toSeq)
     var emptyResponseSeen = false
@@ -442,13 +272,13 @@ class FetchRequestTest extends BaseRequestTest {
   }
 
   private def createTopics(numTopics: Int, numPartitions: Int, configs: Map[String, String] = Map.empty): Map[TopicPartition, Int] = {
-    val topics = (0 until numTopics).map(t => s"topic$t")
+    val topics = (0 until numPartitions).map(t => s"topic$t")
     val topicConfig = new Properties
     topicConfig.setProperty(LogConfig.MinInSyncReplicasProp, 2.toString)
     configs.foreach { case (k, v) => topicConfig.setProperty(k, v) }
     topics.flatMap { topic =>
-      val partitionToLeader = createTopic(topic, numPartitions = numPartitions, replicationFactor = 2,
-        topicConfig = topicConfig)
+      val partitionToLeader = createTopic(zkUtils, topic, numPartitions = numPartitions, replicationFactor = 2,
+        servers = servers, topicConfig = topicConfig)
       partitionToLeader.map { case (partition, leader) => new TopicPartition(topic, partition) -> leader }
     }.toMap
   }
