@@ -21,12 +21,17 @@ import java.util.Properties
 
 import kafka.api.Both
 import kafka.utils.JaasTestUtils.JaasSection
+import org.apache.kafka.common.network.ListenerName
+
 
 class MultipleListenersWithDefaultJaasContextTest extends MultipleListenersWithSameSecurityProtocolBaseTest {
 
-  override def staticJaasSections: Seq[JaasSection] =
-    jaasSections(kafkaServerSaslMechanisms.values.flatMap(identity).toSeq, Some(kafkaClientSaslMechanism), Both)
+  import MultipleListenersWithSameSecurityProtocolBaseTest._
 
-  override protected def dynamicJaasSections: Properties = new Properties
+  override def saslProperties(listenerName: ListenerName): Properties =
+    kafkaClientSaslProperties(Plain, dynamicJaasConfig = true)
+
+  override def jaasSections: Seq[JaasSection] =
+    jaasSections(kafkaServerSaslMechanisms, Some(kafkaClientSaslMechanism), Both)
 
 }
