@@ -17,6 +17,9 @@
 
 package kafka.server
 
+import java.io.File
+
+import org.apache.kafka.common.protocol.SecurityProtocol
 import org.junit.{After, Before, Test}
 import kafka.zk.ZooKeeperTestHarness
 import kafka.utils.TestUtils
@@ -51,11 +54,11 @@ class ReplicaFetchTest extends ZooKeeperTestHarness  {
 
     // create a topic and partition and await leadership
     for (topic <- List(topic1,topic2)) {
-      createTopic(zkClient, topic, numPartitions = 1, replicationFactor = 2, servers = brokers)
+      createTopic(zkUtils, topic, numPartitions = 1, replicationFactor = 2, servers = brokers)
     }
 
     // send test messages to leader
-    val producer = TestUtils.createProducer(TestUtils.getBrokerListStrFromServers(brokers),
+    val producer = TestUtils.createNewProducer(TestUtils.getBrokerListStrFromServers(brokers),
                                                retries = 5,
                                                keySerializer = new StringSerializer,
                                                valueSerializer = new StringSerializer)
